#!/usr/bin/env python3
##
## -----------------------------------------------------------------
##    This file is part of WAPT Software Deployment
##    Copyright (C) 2012 - 2024  Tranquil IT https://www.tranquil.it
##    All Rights Reserved.
##
##    WAPT helps systems administrators to efficiently deploy
##    setup, update and configure applications.
## ------------------------------------------------------------------
##
import logging
import sys
import pyldap
import struct

logger = logging.getLogger()

def check_credentials_ad(conf, username, password, list_dn=[], list_group=[],use_service_account=False):
    """Authenticate against ldap with username and password

    Returns:
        list of groups
    """
    list_group.extend(list_dn)
    primary_group_manage = conf.get("ldap_primary_group_ad_support",True)
    nesting_group_manage = conf.get("ldap_nesting_group_support",True)

    dc_base_dn = conf.get('ldap_auth_base_dn')
    domain_name = conf.get('ldap_domain_name')

    if not domain_name:
        domain_name = dc_base_dn.lower().split('dc=', 1)[-1].replace('dc=', '.').replace(',', '')


    ldap_auth_server = conf.get('ldap_auth_server','')

    auth_ok = {'success': False, 'groups': [], 'error': False, 'msg': ''}

    if not '@' in username:
        full_username = '%s@%s' % (username, domain_name)
    else:
        full_username = username

    if use_service_account:
        bind_username = conf['ldap_account_service_login']
        password = conf['ldap_account_service_password']
    else:
        bind_username = full_username


    try:
        client = pyldap.PyLdapClient(domain_name=domain_name,username=bind_username,password=password,ldap_server_name=ldap_auth_server)
        r = client.bind_sasl_kerberos()
        if not r[0]:
            auth_ok['success'] =False
            auth_ok['msg'] = str(client.result_string())
        else:
            auth_ok['success'] =True
            if dc_base_dn:
                bdn = dc_base_dn
            else:
                bdn = client.root_dn()

            attrneed = ["distinguishedName"]
            dn = ''
            ldap_filter = "(|(sAMAccountName=%s)(userPrincipalName=%s))" % (username,username)
            client.search(bdn, False, ldap_filter, attrneed)

            response =  client.search_result()
            if len(response) > 0:

                attrs = response[0].object_attributes
                attrs_result = { u.name : u.values for u in attrs}
                dn = attrs_result['distinguishedName'][0]

                dn_user = dn
                if primary_group_manage:
                    attrneed.append("objectSID")

                #For real_list_dn search if user member of group
                for group_name in list(list_group):

                    filtertype = 'sAMAccountName'
                    if group_name.lower().startswith('cn='):
                        filtertype = "distinguishedName"

                    if nesting_group_manage:
                        filtergroup = '(&(member:1.2.840.113556.1.4.1941:=%s)(%s=%s))' % (dn_user,filtertype,group_name)
                    else:
                        filtergroup = '(&(member=%s)(%s=%s))' % (dn_user,filtertype,group_name)


                    sidprimaryfilter = ''
                    if primary_group_manage:
                        ldap_filter = '(%s=%s)' % (filtertype,group_name)
                        client.search(bdn, False, ldap_filter, attrneed)
                        response =  client.search_result()
                        if len(response) > 0:
                            attrs = response[0].object_attributes
                            attrs_result = { u.name : u.values for u in attrs}
                            minisid = sid_to_str(attrs_result['objectSid'][0]).rsplit('-')[-1]
                            sidprimaryfilter = '(&(distinguishedName=%s)(primaryGroupID=%s))' % (dn_user,minisid)

                    ldap_filter = '(|%s%s)' % (sidprimaryfilter,filtergroup)
                    logger.debug(ldap_filter)

                    client.search(bdn, False, ldap_filter, attrneed)
                    response =  client.search_result()
                    if len(response) > 0:
                        auth_ok['groups'].append(group_name)
            else:
                logger.debug('Dn username not found')

    except Exception as e:
        auth_ok['success'] = False
        auth_ok['groups'] = []
        auth_ok['error'] = True
        auth_ok['msg'] = str(e)
    finally:
        return auth_ok

def sid_to_str(sid_bytes):
    if isinstance(sid_bytes, str):
        sid_bytes = bytes(sid_bytes, 'latin1')  # ou autre encodage approprié

    revision, sub_authority_count = struct.unpack('BB', sid_bytes[:2])

    identifier_authority = struct.unpack('>Q', b'\x00\x00' + sid_bytes[2:8])[0]

    sub_authorities = struct.unpack('<' + 'I' * sub_authority_count, sid_bytes[8:8 + 4 * sub_authority_count])

    sid = f'S-{revision}-{identifier_authority}'
    for sub_authority in sub_authorities:
        sid += f'-{sub_authority}'

    return sid
